@@ -1,9 +1,3 @@
-"""
-Read/Write AMQP frames over network transports.
-
-2009-01-14 Barry Pederson <bp@barryp.org>
-
-"""
 # Copyright (C) 2009 Barry Pederson <bp@barryp.org>
 #
 # This library is free software; you can redistribute it and/or
@@ -24,6 +18,7 @@ from __future__ import absolute_import
 import errno
 import re
 import socket
+import ssl
 
 # Jython does not have this attribute
 try:
@@ -31,27 +26,18 @@ try:
 except ImportError:  # pragma: no cover
     from socket import IPPROTO_TCP as SOL_TCP  # noqa
 
-#
-# See if Python 2.6+ SSL support is available
-#
 try:
-    import ssl
-    HAVE_PY26_SSL = True
-except:
-    HAVE_PY26_SSL = False
-
-try:
-    bytes
-except:
-    # Python 2.5 and lower
-    bytes = str
+    from ssl import SSLError
+except ImportError:
+    class SSLError(Exception):  # noqa
+        pass
 
 from struct import pack, unpack
 
 from .exceptions import UnexpectedFrame
 from .utils import get_errno, set_cloexec
 
-_UNAVAIL = errno.EAGAIN, errno.EINTR
+_UNAVAIL = errno.EAGAIN, errno.EINTR, errno.ENOENT
 
 AMQP_PORT = 5672
 
@@ -123,9 +109,13 @@ class _AbstractTransport(object):
 
     def __del__(self):
         try:
-            self.close()
-        except socket.error:
-            pass
+            # socket module may have been collected by gc
+            # if this is called by a thread at shutdown.
+            if socket is not None:
+                try:
+                    self.close()
+                except socket.error:
+                    pass
         finally:
             self.sock = None
 
@@ -166,6 +156,10 @@ class _AbstractTransport(object):
         except socket.timeout:
             raise
         except (OSError, IOError, socket.error) as exc:
+            # Don't disconnect for ssl read time outs
+            # http://bugs.python.org/issue10272
+            if isinstance(exc, SSLError) and 'timed out' in str(exc):
+                raise socket.timeout()
             if get_errno(exc) not in _UNAVAIL:
                 self.connected = False
             raise
@@ -200,22 +194,17 @@ class SSLTransport(_AbstractTransport):
         super(SSLTransport, self).__init__(host, connect_timeout)
 
     def _setup_transport(self):
-        """Wrap the socket in an SSL object, either the
-        new Python 2.6 version, or the older Python 2.5 and
-        lower version."""
-        if HAVE_PY26_SSL:
-            if hasattr(self, 'sslopts'):
-                self.sock = ssl.wrap_socket(self.sock, **self.sslopts)
-            else:
-                self.sock = ssl.wrap_socket(self.sock)
-            self.sock.do_handshake()
+        """Wrap the socket in an SSL object."""
+        if hasattr(self, 'sslopts'):
+            self.sock = ssl.wrap_socket(self.sock, **self.sslopts)
         else:
-            self.sock = socket.ssl(self.sock)
+            self.sock = ssl.wrap_socket(self.sock)
+        self.sock.do_handshake()
         self._quick_recv = self.sock.read
 
     def _shutdown_transport(self):
         """Unwrap a Python 2.6 SSL socket, so we can call shutdown()"""
-        if HAVE_PY26_SSL and self.sock is not None:
+        if self.sock is not None:
             try:
                 unwrap = self.sock.unwrap
             except AttributeError:
@@ -232,7 +221,7 @@ class SSLTransport(_AbstractTransport):
         try:
             while len(rbuf) < n:
                 try:
-                    s = recv(131072)  # see note above
+                    s = recv(n - len(rbuf))  # see note above
                 except socket.error as exc:
                     # ssl.sock.read may cause ENOENT if the
                     # operation couldn't be performed (Issue celery#1414).
@@ -250,12 +239,17 @@ class SSLTransport(_AbstractTransport):
 
     def _write(self, s):
         """Write a string out to the SSL socket fully."""
-        write = self.sock.write
-        while s:
-            n = write(s)
-            if not n:
-                raise IOError('Socket closed')
-            s = s[n:]
+        try:
+            write = self.sock.write
+        except AttributeError:
+            # Works around a bug in python socket library
+            raise IOError('Socket closed')
+        else:
+            while s:
+                n = write(s)
+                if not n:
+                    raise IOError('Socket closed')
+                s = s[n:]
 
 
 class TCPTransport(_AbstractTransport):
@@ -275,7 +269,7 @@ class TCPTransport(_AbstractTransport):
         try:
             while len(rbuf) < n:
                 try:
-                    s = recv(131072)
+                    s = recv(n - len(rbuf))
                 except socket.error as exc:
                     if not initial and exc.errno in _errnos:
                         continue
